# -*- coding: utf-8 -*-
import mod.client.extraClientApi as clientApi
import time

import config

CompFactory = clientApi.GetEngineCompFactory()
gameComp = CompFactory.CreateGame(clientApi.GetLevelId())


class TutorialClientSystem(clientApi.GetClientSystemCls()):

    def __init__(self, namespace, name):
        super(TutorialClientSystem, self).__init__(namespace, name)
        self.ListenEvent()
        self.mTimeCounter = 0
        self.mCustomGunUINode = None  # 自定义枪械的 UI Node
        self.mCustomFireBottleUINode = None  # 燃烧瓶的 UI Node
        #
        self.mQueryVariableComp = CompFactory.CreateQueryVariable(clientApi.GetLocalPlayerId())
        self.mItemComp = CompFactory.CreateItem(clientApi.GetLocalPlayerId())
        #
        self.mCarriedItem = self.mItemComp.GetCarriedItem()  # 手持物品
        self.mAttackStep = 0  # 攻击的阶段
        self.mLastAttackTime = 0  # 上一次攻击的时间
        #
        self.mIsUsingItem = False  # 是否正在使用投掷类武器
        self.mStartUsingFrame = 0  # 开始使用物品的时间

    def ListenEvent(self):
        # 自定义事件
        self.ListenForEvent('tutorialMod', 'tutorialServerSystem', 'SyncAttackStateEvent', self, self.OnSyncAttackStateEvent)
        self.ListenForEvent('tutorialMod', 'tutorialServerSystem', 'SyncCustomGunStateEvent', self, self.OnSyncCustomGunStateEvent)
        self.ListenForEvent('tutorialMod', 'tutorialServerSystem', 'SyncCustomFireBottleStateEvent', self,
                            self.OnSyncCustomFireBottleStateEvent)
        # 系统事件
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "UiInitFinished",
                            self, self.OnUiInitFinished)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "AddPlayerCreatedClientEvent",
                            self, self.OnAddPlayerCreatedClientEvent)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "OnKeyPressInGame",
                            self, self.OnOnKeyPressInGame)
        # 处理攻击相关的事件，监听左键按下事件
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "TapBeforeClientEvent",
                            self, self.OnLeftClick)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "LeftClickBeforeClientEvent",
                            self, self.OnLeftClick)
        # 投掷武器相关的事件
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "ClientItemTryUseEvent", self,
                            self.OnClientItemTryUseEvent)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "ItemReleaseUsingClientEvent", self,
                            self.OnItemReleaseUsingClientEvent)

    def UnListenEVent(self):
        # 自定义事件
        self.UnListenForEvent('tutorialMod', 'tutorialServerSystem', 'SyncAttackStateEvent', self, self.OnSyncAttackStateEvent)
        self.UnListenForEvent('tutorialMod', 'tutorialServerSystem', 'SyncCustomGunStateEvent', self, self.OnSyncCustomGunStateEvent)
        # 系统事件
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "UiInitFinished",
                              self, self.OnUiInitFinished)
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "AddPlayerCreatedClientEvent",
                              self, self.OnAddPlayerCreatedClientEvent)
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "OnKeyPressInGame",
                              self, self.OnOnKeyPressInGame)

    def Destroy(self):
        self.UnListenEVent()

    def Update(self):
        self.mTimeCounter += 1

    # 接受到其他客户端传来的同步事件
    def OnSyncAttackStateEvent(self, args):
        playerId = args['playerId']
        value = float(args['value'])
        CompFactory.CreateQueryVariable(playerId).Set(config.CustomWeaponAttackVarName, value)

    def OnSyncCustomGunStateEvent(self, args):
        playerId = args['playerId']
        value = float(args['value'])
        CompFactory.CreateQueryVariable(playerId).Set(config.CustomGunAttackVarName, value)

    def OnSyncCustomFireBottleStateEvent(self, args):
        playerId = args['playerId']
        value = float(args['value'])
        CompFactory.CreateQueryVariable(playerId).Set(config.FireBottleAttackVarName, value)

    def OnUiInitFinished(self, args=None):
        self.mCarriedItem = self.mItemComp.GetCarriedItem()  # 手持物品
        # 注册 UI
        clientApi.RegisterUI('tutorialMod', 'tutorialCustomGunUI', 'tutorialScripts.uiScripts.UIScripts', 'tutorialGunUI.main')
        self.mCustomGunUINode = clientApi.CreateUI('tutorialMod', 'tutorialCustomGunUI', {'isHud': 1})
        # 注册燃烧瓶 UI
        clientApi.RegisterUI('tutorialMod', 'tutorialCustomFireBottleUI', 'tutorialScripts.fireBottleUIScripts.FireBottleUIScripts',
                             'tutorialFireBottleUI.main')
        self.mCustomFireBottleUINode = clientApi.CreateUI('tutorialMod', 'tutorialCustomFireBottleUI', {'isHud': 1})

    def OnAddPlayerCreatedClientEvent(self, args):
        playerId = args['playerId']
        self.InitRender(playerId)  # 包括其他玩家也需要被初始化

    def OnOnKeyPressInGame(self, args):
        screenName = args['screenName']
        key = int(args['key'])
        isDown = int(args['isDown'])
        if isDown:
            if key == clientApi.GetMinecraftEnum().KeyBoardType.KEY_TAB:
                self.NotifyToServer("SwitchGameMode", {"playerId": clientApi.GetLocalPlayerId()})
            elif key == clientApi.GetMinecraftEnum().KeyBoardType.KEY_I:
                comp = clientApi.GetEngineCompFactory().CreateSkyRender(clientApi.GetLevelId())
                comp.ResetSkyColor()

    def OnLeftClick(self, args=None):
        # 手持自定义武器并且上一次的攻击快要结束时，才会响应攻击
        if self._IsCarriedCustomWeapon():
            args['cancel'] = True  # 不响应原版的点击事件
            if self._LastAttackWillFinished():
                self._HandleAttack()

    def OnClientItemTryUseEvent(self, args):
        if args['itemName'] == 'tutorial:custom_throw_weapon':
            self.mIsUsingItem = True
            self.mStartUsingFrame = self.mTimeCounter

    def OnItemReleaseUsingClientEvent(self, args):
        itemDict = args['itemDict']
        if itemDict and itemDict['itemName'] == 'tutorial:custom_throw_weapon':
            # 最大蓄力 2s，也就是说威力最多为 2 倍
            power = min(2.0, (self.mTimeCounter - self.mStartUsingFrame) / 30.0)
            self.NotifyToServer('ThrowCustomWeapon', {'playerId': args['playerId'], 'power': power})

    # 初始化绑定
    def InitRender(self, playerId):
        queryVariableComp = CompFactory.CreateQueryVariable(playerId)

        # 定义攻击的自定义变量
        queryVariableComp.Register(config.CustomWeaponAttackVarName, 0)
        # 第一种方法：作为玩家的附加骨骼添加进渲染控制器中进行控制
        self._InitMethodOne(playerId)

        # 另外一种形式的 3D 武器：附加骨骼
        # 则是通过 query.item_slot_to_bone_name 把模型绑定上了固定的位置，再用动画进行修正

        # 自定义枪械
        self._InitToCustomGun(playerId)
        # 燃烧瓶
        self._InitToFireBottle(playerId)
        # 投掷武器
        self._InitToThrowWeapon(playerId)

        # 两种方式都通用的第三人称动画和控制器
        actorRenderComp = CompFactory.CreateActorRender(playerId)
        actorRenderComp.AddPlayerAnimation("third_person_attack1", "animation.tutorial_custom_sword_by_attachable.player.attack1")
        actorRenderComp.AddPlayerAnimationController("custom_sword_third_attack", "controller.animation.custom_sword.third_attack")
        actorRenderComp.AddPlayerScriptAnimate("custom_sword_third_attack", "!variable.is_first_person")
        actorRenderComp.RebuildPlayerRender()

    # 为第一个方法进行初始化渲染
    def _InitMethodOne(self, playerId):
        actorRenderComp = CompFactory.CreateActorRender(playerId)
        # 3D武器：额外骨骼 —— 所需的就是把骨骼作为玩家的一部分添加上
        actorRenderComp.AddPlayerGeometry("custom_sword", "geometry.tutorial_custom_sword_by_addon_bones")
        actorRenderComp.AddPlayerTexture("custom_sword", "textures/models/tutorial_custom_sword")
        actorRenderComp.AddPlayerRenderController("controller.render.tutorial_custom_sword",
                                                  "query.get_equipped_item_name('main_hand') == 'custom_sword_by_addon_bones'")
        # 作为额外骨骼，还需要添加第一人称的动画文件和相关的控制器
        actorRenderComp.AddPlayerAnimation("custom_sword_first_hold", "animation.tutorial_custom_sword_by_addon_bones.hold_first_person")
        actorRenderComp.AddPlayerAnimation("custom_sword_first_attack", "animation.tutorial_custom_sword_by_addon_bones.first_attack1")
        actorRenderComp.AddPlayerAnimationController("custom_sword_first_attack_controller",
                                                     "controller.animation.custom_sword_by_addon_bones")
        actorRenderComp.AddPlayerScriptAnimate(
            "custom_sword_first_hold",
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_sword_by_addon_bones'"
        )
        actorRenderComp.AddPlayerScriptAnimate(
            "custom_sword_first_attack_controller",
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_sword_by_addon_bones'"
        )

    # 自定义枪械初始化渲染
    def _InitToCustomGun(self, playerId):
        queryVariableComp = CompFactory.CreateQueryVariable(playerId)
        queryVariableComp.Register(config.CustomGunAttackVarName, 0)
        queryVariableComp.Set(config.CustomGunAttackVarName, 0)
        # 射速相关的定义
        queryVariableComp.Register(config.CustomGunAttackSpeedVarName, config.CustomGunDefaultAttackSpeed)
        queryVariableComp.Set(config.CustomGunAttackSpeedVarName, config.CustomGunDefaultAttackSpeed)

        actorRenderComp = CompFactory.CreateActorRender(playerId)
        actorRenderComp.AddPlayerGeometry('custom_gun', 'geometry.tutorial_custom_gun')
        actorRenderComp.AddPlayerTexture('custom_gun', 'textures/models/tutorial_custom_gun')
        actorRenderComp.AddPlayerRenderController("controller.render.tutorial_custom_gun",
                                                  "query.get_equipped_item_name('main_hand') == 'custom_gun'")

        # 定义动画和控制器名称
        animations = ['hold_first_person', 'attack_first_person', 'attack_third_person']
        controllers = ['custom_gun_first_person', 'custom_gun_third_person']

        for anim in animations:
            animationKey = 'custom_gun_' + anim
            animationName = 'animation.tutorial_custom_gun.' + anim
            actorRenderComp.AddPlayerAnimation(animationKey, animationName)

        for controller in controllers:
            controllerKey = controller + "_controller"
            controllerName = 'controller.animation.' + controller
            actorRenderComp.AddPlayerAnimationController(controllerKey, controllerName)

        # 添加动画的触发条件
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_gun_hold_first_person',
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_gun'"
        )
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_gun_first_person_controller',
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_gun'"
        )
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_gun_third_person_controller',
            "!variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_gun'"
        )

        # 添加特效
        actorRenderComp.AddPlayerParticleEffect('gun_fire', 'tutorial:gun_fire')
        # 有兴趣的同学也可以尝试加一下音效

    # 燃烧瓶渲染器
    def _InitToFireBottle(self, playerId):
        queryVariableComp = CompFactory.CreateQueryVariable(playerId)
        queryVariableComp.Register(config.FireBottleAttackVarName, 0)
        queryVariableComp.Set(config.FireBottleAttackVarName, 0)

        actorRenderComp = CompFactory.CreateActorRender(playerId)
        # 控制器
        actorRenderComp.AddPlayerGeometry('custom_fire_bottle', 'geometry.tutorial_custom_fire_bottle')
        actorRenderComp.AddPlayerTexture('custom_fire_bottle', 'textures/models/tutorial_custom_fire_bottle')
        actorRenderComp.AddPlayerRenderController("controller.render.tutorial_custom_fire_bottle",
                                                  "query.get_equipped_item_name('main_hand') == 'custom_fire_bottle'")
        # 定义动画和控制器名称
        animations = ['hold_first_person', 'attack_first_person', 'attack_third_person']
        controllers = ['custom_fire_bottle_first_person', 'custom_fire_bottle_third_person']

        for anim in animations:
            animationKey = 'custom_fire_bottle_' + anim
            animationName = 'animation.tutorial_custom_fire_bottle.' + anim
            actorRenderComp.AddPlayerAnimation(animationKey, animationName)

        for controller in controllers:
            controllerKey = controller + "_controller"
            controllerName = 'controller.animation.' + controller
            actorRenderComp.AddPlayerAnimationController(controllerKey, controllerName)

        # 添加动画的触发条件
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_fire_bottle_hold_first_person',
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_fire_bottle'"
        )
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_fire_bottle_first_person_controller',
            "variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_fire_bottle'"
        )
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_fire_bottle_third_person_controller',
            "!variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_fire_bottle'"
        )

    # 投掷武器的渲染
    def _InitToThrowWeapon(self, playerId):
        actorRenderComp = CompFactory.CreateActorRender(playerId)
        # 使用原版的动画
        actorRenderComp.AddPlayerAnimation('custom_throw_weapon_third_person_raise', 'animation.humanoid.brandish_spear')
        actorRenderComp.AddPlayerScriptAnimate(
            'custom_throw_weapon_third_person_raise',
            "!variable.is_first_person && query.get_equipped_item_name('main_hand') == 'custom_throw_weapon' && query.main_hand_item_use_duration > 0"
        )

    def _IsCarriedCustomWeapon(self):
        self.mCarriedItem = self.mItemComp.GetCarriedItem()
        return self.mCarriedItem and self.mCarriedItem['itemName'] in config.CustomWeaponList

    def _LastAttackWillFinished(self):
        currentTime = time.time()
        return currentTime - config.AttackInterval > self.mLastAttackTime

    def _HandleAttack(self):
        self.mLastAttackTime = time.time()
        gameComp.AddTimer(0, self._SetAttackStateAndSyncToOtherClients, 'start')
        gameComp.AddTimer(0.2, self._SetAttackStateAndSyncToOtherClients, 'will_hit')
        gameComp.AddTimer(0.5, self._SetAttackStateAndSyncToOtherClients, 'end')

    def _SetAttackStateAndSyncToOtherClients(self, state):
        # 设置本地的自定义变量
        self.mQueryVariableComp.Set(config.CustomWeaponAttackVarName, 1.0 if state in ['start', 'will_hit'] else 0)
        # 通知其他客户端更新数据
        self.NotifyToServer("SyncAttackStateEvent", {'playerId': clientApi.GetLocalPlayerId(), 'state': state})
