# -*- coding: utf-8 -*-
#
import math
import mod.server.extraServerApi as serverApi
from mod.common.utils.mcmath import Vector3

CompFactory = serverApi.GetEngineCompFactory()
gameComp = CompFactory.CreateGame(serverApi.GetLevelId())


class TutorialServerSystem(serverApi.GetServerSystemCls()):
    def __init__(self, namespace, name):
        super(TutorialServerSystem, self).__init__(namespace, name)
        self.ListenEvent()
        #
        self.mTimeCounter = 0

    def Update(self):
        self.mTimeCounter += 1

    def ListenEvent(self):
        # 自定义事件
        self.ListenForEvent('tutorialMod', 'tutorialClientSystem', "SwitchGameMode", self, self.OnSwitchGameMode)
        self.ListenForEvent('tutorialMod', 'tutorialClientSystem', "SyncAttackStateEvent", self, self.OnSyncAttackStateEvent)
        self.ListenForEvent('tutorialMod', 'tutorialClientSystem', "SyncCustomGunAttackStateEvent", self,
                            self.OnSyncCustomGunAttackStateEvent)
        self.ListenForEvent('tutorialMod', 'tutorialClientSystem', "SyncCustomFireBottleStateEvent", self,
                            self.OnSyncCustomFireBottleStateEvent)
        self.ListenForEvent('tutorialMod', 'tutorialClientSystem', "ThrowCustomWeapon", self, self.OnThrowCustomWeapon)

        # 系统事件
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ProjectileDoHitEffectEvent", self,
                            self.OnProjectileDoHitEffectEvent)

    def UnListenEvent(self):
        # 自定义事件
        self.UnListenForEvent('tutorialMod', 'tutorialClientSystem', "SwitchGameMode", self, self.OnSwitchGameMode)
        self.UnListenForEvent('tutorialMod', 'tutorialClientSystem', "SyncAttackStateEvent", self, self.OnSyncAttackStateEvent)
        self.UnListenForEvent('tutorialMod', 'tutorialClientSystem', "SyncCustomGunAttackStateEvent", self,
                              self.OnSyncCustomGunAttackStateEvent)

        # 系统事件
        self.UnListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ProjectileDoHitEffectEvent", self,
                              self.OnProjectileDoHitEffectEvent)

    def Destroy(self):
        self.UnListenEvent()

    # region 监听事件
    # --------------------------------------------------------------------------------------------
    def OnSwitchGameMode(self, args):
        playerId = args['playerId']
        playerComp = CompFactory.CreatePlayer(playerId)
        gameType = CompFactory.CreateGame(serverApi.GetLevelId()).GetPlayerGameType(playerId)
        if gameType != serverApi.GetMinecraftEnum().GameType.Survival:
            setGameType = serverApi.GetMinecraftEnum().GameType.Survival
        else:
            setGameType = serverApi.GetMinecraftEnum().GameType.Creative
        playerComp.SetPlayerGameType(setGameType)

    def OnSyncAttackStateEvent(self, args):
        playerId = args['playerId']
        state = args['state']
        if state == 'will_hit':
            weaponDamage = self._GetCarriedWeaponDamage(playerId)
            self._HurtFrontArea(playerId, 3, weaponDamage)
        else:
            relevantPlayers = CompFactory.CreatePlayer(playerId).GetRelevantPlayer([playerId])
            self.NotifyToMultiClients(relevantPlayers, 'SyncAttackStateEvent', {
                'playerId': playerId,
                'value'   : 1.0 if state == 'start' else 0.0
            })

    def OnSyncCustomGunAttackStateEvent(self, args):
        playerId = args['playerId']
        state = args['state']
        if state == 'fire':
            self._ShotBullet(playerId)
        else:
            relevantPlayers = CompFactory.CreatePlayer(playerId).GetRelevantPlayer([playerId])
            self.NotifyToMultiClients(relevantPlayers, 'SyncCustomGunStateEvent', {
                'playerId': playerId,
                'value'   : 1.0 if state == 'start' else 0.0
            })

    def OnSyncCustomFireBottleStateEvent(self, args):
        playerId = args['playerId']
        state = args['state']
        if state == 'throw':
            self._ThrowFireBottle(playerId)
        else:
            relevantPlayers = CompFactory.CreatePlayer(playerId).GetRelevantPlayer([playerId])
            self.NotifyToMultiClients(relevantPlayers, 'SyncCustomFireBottleStateEvent', {
                'playerId': playerId,
                'value'   : 1.0 if state == 'start' else 0.0
            })

    def OnThrowCustomWeapon(self, args):
        playerId = args['playerId']
        power = args['power']

        param = {
            'power'    : 4 * power,
            'damage'   : 3 + power * 3,  # 最高伤害为 3+2*3=9 点伤害
            'direction': serverApi.GetDirFromRot(CompFactory.CreateRot(playerId).GetRot())
        }
        projectileComp = CompFactory.CreateProjectile(playerId)
        projectileEntityId = projectileComp.CreateProjectileEntity(playerId, 'tutorial:thrown_custom_throw_weapon', param)

        if projectileEntityId != '-1':
            self._ReduceCarriedItemNum(playerId, 1)

    def OnProjectileDoHitEffectEvent(self, args):
        projectileId = args['id']
        hitTargetType = args['hitTargetType']
        identifier = CompFactory.CreateEngineType(projectileId).GetEngineTypeStr()

    # endregion

    # region 类函数
    # --------------------------------------------------------------------------------------------
    def _GetCarriedWeaponDamage(self, playerId):
        itemComp = CompFactory.CreateItem(playerId)
        carriedItem = itemComp.GetPlayerItem(serverApi.GetMinecraftEnum().ItemPosType.CARRIED, 0)
        basicInfo = itemComp.GetItemBasicInfo(carriedItem['itemName'])
        return basicInfo['weaponDamage']

    # 攻击前方的区域
    def _HurtFrontArea(self, attacker, radius, damage, betweenAngle=75, knocked=True):
        attackerPos = CompFactory.CreatePos(attacker).GetFootPos()
        attackerRot = CompFactory.CreateRot(attacker).GetRot()
        # 计算攻击者朝向向量
        forwardVector = serverApi.GetDirFromRot(attackerRot)

        entityList = gameComp.GetEntitiesAround(attacker, 6, {
            'any_of': {
                'test'    : 'is_family',
                'subject' : 'other',
                'operator': 'not',
                'value'   : 'instabuild'
            }
        })

        for _entityId in entityList:
            entityPos = CompFactory.CreatePos(_entityId).GetFootPos()
            delta = Vector3(entityPos) - Vector3(attackerPos)

            # 计算角度
            angle = math.degrees(math.acos(Vector3.Dot(delta.Normalized(), Vector3(forwardVector).Normalized())))

            # 判断是否在扇形攻击范围内
            if betweenAngle == 0.0 or (angle < betweenAngle and delta.Length() < radius):
                CompFactory.CreateHurt(_entityId).Hurt(
                    damage, serverApi.GetMinecraftEnum().ActorDamageCause.EntityAttack, attacker, knocked=knocked
                )

    # 发射子弹
    def _ShotBullet(self, playerId):
        rot = CompFactory.CreateRot(playerId).GetRot()
        footPos = CompFactory.CreatePos(playerId).GetFootPos()
        shotPos = (footPos[0], footPos[1] + 1.25, footPos[2])
        shotPos = self._MoveForward(shotPos, rot, 1.3)
        param = {
            'power'    : 1.2,
            'gravity'  : 0,
            'position' : shotPos,
            'direction': serverApi.GetDirFromRot(rot)
        }

        projectileComp = CompFactory.CreateProjectile(playerId)
        projectileComp.CreateProjectileEntity(playerId, 'tutorial:custom_gun_bullet_projectile', param)

        self._ReduceItem(playerId, 'tutorial:custom_gun_bullet', 1)

    # 向前投掷燃烧瓶
    def _ThrowFireBottle(self, playerId):
        rot = CompFactory.CreateRot(playerId).GetRot()
        # 默认是向上抬一点
        rot = (rot[0] - 30, rot[1])
        param = {
            'power'    : 1.2,
            'gravity'  : 0.125,
            'direction': serverApi.GetDirFromRot(rot)
        }

        projectileComp = CompFactory.CreateProjectile(playerId)
        projectileComp.CreateProjectileEntity(playerId, 'tutorial:custom_fire_bottle_projectile', param)

        self._ReduceCarriedItemNum(playerId, 1)

    # 向前移动坐标
    def _MoveForward(self, pos, rot, distance):
        _rot = (0, rot[1])  # 第一个参数是上下角度，第二个是左右角度
        rx, ry, rz = serverApi.GetDirFromRot(_rot)
        return (pos[0] + rx * distance, pos[1] + ry * distance, pos[2] + rz * distance)

    def _ReduceItem(self, playerId, itemName, count):
        itemComp = CompFactory.CreateItem(playerId)
        allItems = itemComp.GetPlayerAllItems(serverApi.GetMinecraftEnum().ItemPosType.INVENTORY)
        remainingCount = count  # 剩余要减少的数量

        for slotPos, itemDict in enumerate(allItems):
            if itemDict and itemDict['itemName'] == itemName:
                item_count = itemDict['count']
                if item_count >= remainingCount:
                    itemComp.SetInvItemNum(slotPos, item_count - remainingCount)
                    return True  # 减少成功
                else:
                    itemComp.SetInvItemNum(slotPos, 0)
                    remainingCount -= item_count

        return False  # 减少失败，物品数量不足

    def _ReduceCarriedItemNum(self, playerId, reduceNum):
        itemComp = CompFactory.CreateItem(playerId)
        selectSlotId = itemComp.GetSelectSlotId()
        itemDict = itemComp.GetPlayerItem(serverApi.GetMinecraftEnum().ItemPosType.INVENTORY, selectSlotId)
        return itemComp.SetInvItemNum(selectSlotId, itemDict['count'] - reduceNum)

    # endregion
