# -*- coding: utf-8 -*-
import mod.client.extraClientApi as clientApi

import config

ViewBinder = clientApi.GetViewBinderCls()
ViewRequest = clientApi.GetViewViewRequestCls()
ScreenNode = clientApi.GetScreenNodeCls()

Namespace = clientApi.GetEngineNamespace()
SystemName = clientApi.GetEngineSystemName()
CompFactory = clientApi.GetEngineCompFactory()
gameComp = CompFactory.CreateGame(clientApi.GetLevelId())


#ui布局绑定
class UIScripts(ScreenNode):
    def __init__(self, namespace, name, param):
        ScreenNode.__init__(self, namespace, name, param)
        self.mPlayerId = clientApi.GetLocalPlayerId()
        self.mClientSystem = clientApi.GetSystem('tutorialMod', 'tutorialClientSystem')
        self.mItemComp = CompFactory.CreateItem(self.mPlayerId)
        self.mQueryVariableComp = CompFactory.CreateQueryVariable(self.mPlayerId)

        # 组件地址
        self.mShotBtnPath = "/shotBtn"
        self.mBulletLabelPath = "/bulletNumLabel"

        # 界面需要使用的自定义属性
        self.mFrameCnt = 0
        self.mCarriedItem = self.mItemComp.GetCarriedItem()  # 手持物品
        self.mIsBtnDown = False  # 按钮是否被按下
        self.mBtnDownFrame = 0  # 按钮按下帧数
        self.mShotSpeed = 0  # 攻击速度
        self.mAttackFrame = 0  # 攻击所需要的帧数
        self.mBulletNum = 0  # 子弹数量

    def Create(self):
        print("===== Tutorial Custom Gun UI Create Finished =====")
        # 注册按钮的事件
        control = self.GetBaseUIControl(self.mShotBtnPath).asButton()
        control.AddTouchEventParams({"isSwallow": True})
        control.SetButtonTouchDownCallback(self.OnShotBtnDown)
        control.SetButtonTouchUpCallback(self.OnShotBtnUp)

        # 关注事件
        namespace, systemName = clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName()
        self.mClientSystem.ListenForEvent(namespace, systemName, "OnCarriedNewItemChangedClientEvent", self, self.OnCarriedNewItem)

        # 刚创建时也自动触发一次
        self.OnCarriedNewItem({'itemDict': self.mItemComp.GetCarriedItem()})

    def Update(self):
        self.mFrameCnt += 1
        if self.mBtnDownFrame > 0:
            if self.mBtnDownFrame + self.mAttackFrame <= self.mFrameCnt:
                if self.mIsBtnDown:
                    # 还处于按下的状态，那么继续攻击
                    self.mBtnDownFrame = self.mFrameCnt + self.mAttackFrame
                    self._HandleAttack()
                else:
                    # 还原 query 变量
                    self.mQueryVariableComp.Set(config.CustomGunAttackVarName, 0.0)

    # region 按钮事件
    # --------------------------------------------------------------------------------------------
    def OnShotBtnDown(self, args):
        self._FreshBagBullet()
        if self.mBulletNum == 0:
            # 没有子弹不响应
            return
        self.mIsBtnDown = True
        if self.mFrameCnt >= self.mBtnDownFrame + self.mAttackFrame:
            self.mBtnDownFrame = self.mFrameCnt
            self._HandleAttack()

    def OnShotBtnUp(self, args):
        self.mIsBtnDown = False

    # endregion

    # region 事件监听
    # --------------------------------------------------------------------------------------------
    def OnCarriedNewItem(self, args):
        self.mCarriedItem = args['itemDict']
        if self._IsCarriedCustomGun():
            self._SetUIVisible(True)
            self._FreshUIData()
        else:
            self._SetUIVisible(False)

    # endregion

    # region 类函数
    # --------------------------------------------------------------------------------------------
    def _IsCarriedCustomGun(self):
        if self.mCarriedItem and self.mCarriedItem['itemName'] == 'tutorial:custom_gun':
            return True
        return False

    def _FreshUIData(self):
        # 读取攻击速度
        self.mShotSpeed = self.mQueryVariableComp.Get(config.CustomGunAttackSpeedVarName)
        self.mAttackFrame = self._GetAttackFrame()
        # 读取背包的信息，查看子弹信息
        self._FreshBagBullet()

    def _FreshBagBullet(self):
        bulletNum = 0
        allItems = self.mItemComp.GetPlayerAllItems(clientApi.GetMinecraftEnum().ItemPosType.INVENTORY)
        for _itemDict in allItems:
            if not _itemDict:
                continue
            if _itemDict and _itemDict['itemName'] == 'tutorial:custom_gun_bullet':
                bulletNum += _itemDict['count']
        self.mBulletNum = bulletNum
        self.GetBaseUIControl(self.mBulletLabelPath).asLabel().SetText("背包剩余子弹:" + str(self.mBulletNum))

    def _SetUIVisible(self, flag):
        self.SetScreenVisible(flag)

    def _GetAttackFrame(self):
        framePerSecond = 30
        return int(config.CustomGunAttackAnimDuration * (1 / float(self.mShotSpeed)) * framePerSecond)

    def _HandleAttack(self):
        animTotalTime = config.CustomGunAttackAnimDuration * (1 / float(self.mShotSpeed))
        # 原版的的等式转换一下而已
        # fireTime / float(animTotalTime) = config.CustomGunAttackAnimAttackFrame / float(config.CustomGunAttackAnimDuration)
        fireTime = config.CustomGunAttackAnimAttackFrame / float(config.CustomGunAttackAnimDuration) * float(animTotalTime)

        gameComp.AddTimer(0, self._SetAttackStateAndSyncToOtherClients, 'start')
        gameComp.AddTimer(fireTime, self._SetAttackStateAndSyncToOtherClients, 'fire')
        gameComp.AddTimer(animTotalTime, self._SetAttackStateAndSyncToOtherClients, 'end')

        # 刷新背包的子弹数
        gameComp.AddTimer(animTotalTime, self._FreshBagBullet)

    def _SetAttackStateAndSyncToOtherClients(self, state):
        # 设置本地自定义变量
        self.mQueryVariableComp.Set(config.CustomGunAttackVarName, 1.0 if state in ['start', 'fire'] else 0.0)
        # 通知其他客户端
        self.mClientSystem.NotifyToServer('SyncCustomGunAttackStateEvent', {'state': state, 'playerId': self.mPlayerId})

    # endregion
